%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Example Script for Consistent Wiener Filtering
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2013 Emmanuel Vincent and Jonathan Le Roux
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

data='./data/';
wlen=1024;
hop=512;
files={'Sq_Ce_A'};
snrs=[-10 0 10];

m1 = 1; % file
m2 = 1; % SNR;       1: -10 dB, 2: 0 dB, 3: +10 dB
m3 = 1; % Variance;  1: Oracle, 2: Spectral subtraction
m4 = 1; % algorithm; 1: Conjugate gradient with soft penalty;
        %            2: Conjugate gradient with hard constraint;

% Create the mixture
file=[data 'Dev_2ch_1src_' files{m1}];
snr=snrs(m2);
s=wavread([file '_Src.wav']).';
n=wavread([file '_Noise.wav']).';
n=sum(n);
nsampl=length(s);
s=10^(snr/20)*s/sqrt(sum(s.^2)/nsampl);
n=n/sqrt(sum(n.^2)/nsampl);
x=s+n;

% Compute the variances
if m3==1, % oracle
    VS=abs(stft(s,wlen,hop)).^2;
    VN=abs(stft(n,wlen,hop)).^2;
else % power subtraction
    VX=abs(stft(x,wlen,hop)).^2;
    VN=abs(stft(n,wlen,hop)).^2;
    VN=repmat(mean(VN,2),[1 size(VX,2)]);
    VS=max(0,VX-VN);
end

% Apply the consistent Wiener filter
opts = struct;
opts.hop = hop;
if m4==1,
    opts.type='penalty';
    opts.gamma = 1e5;
else
    opts.type='equality';
end

[SE,se,iter] = Consistent_Wiener_Filter(x,VS,VN,opts);
